import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A Slime. Slime lives in Dungeons. They like to eat Humans. (Well, in our game
 * they do...)
 * 
 * Version: 2
 * 
 * The Slime walks around randomly. If it runs into a human it eats it.
 * In this version, we have added a sound effect, and the game stops when
 * a Slime eats the human.
 */

public class Slime extends Monster
{
    private GreenfootImage image1;
    private GreenfootImage image2;
    private int vSpeed = 0;
    private int acceleration = 2;
    private int speed = 2;
    
    /**
     * Do whatever Slimes do.
     */
    public void act()
    {
        checkWallAbove();
        checkWallBelow();
        checkRightWalls();
        checkLeftWalls();
        randomTurn();
        move();
        lookForDungeonMaster();
    }
    
      
/**
* Check if there is a wall above Slime.
*/
public boolean checkWallAbove()
{
    int spriteHeight = getImage().getHeight();
    int yDistance = (int) (spriteHeight / -2);

    Actor ceiling = getOneObjectAtOffset (0, yDistance, HWall.class);
    if(ceiling != null)
    {
        vSpeed =1;
        bopHead(ceiling);
        return true;
    } 
    else
    {

        return false;
    }

}

public void bopHead(Actor ceiling)
{
    int ceilingHeight = ceiling.getImage().getHeight();
    int newY = ceiling.getY() + (ceilingHeight + getImage().getHeight() /2);

    setLocation(getX(), newY);  
}

/**
* Move to Ground Command to work in conjuction with checkWallBelow.
*/
public void moveToGround(Actor ground)
{
    int groundHeight = ground.getImage().getHeight();
    int newY = ground.getY() - (groundHeight + getImage().getHeight())/2;
    
    setLocation(getX(),newY);
}

/**
* Check if there is a wall below Slime.
*/
public boolean checkWallBelow()
{
    int spriteHeight = getImage().getHeight();
    int yDistance = (int) (spriteHeight / -2 + 5);

    Actor floor = getOneObjectAtOffset (0, getImage().getHeight()/2, HWall.class);
    if(floor == null)
    {
        return false;
    } 
    else
    {
        moveToGround(floor);
        return true;
    }

}

public void bopFeet(Actor floor)
{
    int floorHeight = floor.getImage().getHeight();
    int newY = floor.getY() - (floorHeight + getImage().getHeight() /2);

    setLocation(getX(), newY);  
}

 /**
     * Check if there is a wall to the right of Slime.
     */
    public boolean checkRightWalls()
{
    int spriteWidth = getImage().getWidth();
    int xDistance = (int) (spriteWidth/2);
    
    Actor rightWall = getOneObjectAtOffset (xDistance, 0, VWall.class);
    
   if(rightWall == null)
   {
       return false;
   }
   else
   {
       stopByRightWall(rightWall);
       return true;
   }
}

public void stopByRightWall (Actor rightWall)
{
    int wallWidth = rightWall.getImage().getWidth();
    int newX = rightWall.getX() -(wallWidth + getImage().getWidth())/2;
    
    setLocation(newX -5, getY());
}

 /**
 * Check if there is a wall to the left of Slime.
 */
    public boolean checkLeftWalls()
{
    int spriteWidth = getImage().getWidth();
    int xDistance = (int) (spriteWidth/2);
    
    Actor leftWall = getOneObjectAtOffset (-xDistance, 0, VWall.class);
    
   if(leftWall == null)
   {
       return false;
   }
   else
   {
       stopByLeftWall(leftWall);
       return true;
   }
}

public void stopByLeftWall (Actor leftWall)
{
    int wallWidth = leftWall.getImage().getWidth();
    int newX = leftWall.getX() +(wallWidth + getImage().getWidth())/2;
    
    setLocation(newX +5, getY());
}

    /**
     * Randomly decide to turn from the current direction, or not. If we turn
     * turn a bit left or right by a random degree.
     */
    public void randomTurn()
    {
        if (Greenfoot.getRandomNumber(100) > 90) {
            turn(Greenfoot.getRandomNumber(90)-45);
        }
    }

    /**
     * Try to eat a DungeonMaster. That is: check whether we have stumbled upon a DungeonMaster.
     * If we have, remove the DungeonMaster from the game, and stop the program running.
     */
    public void lookForDungeonMaster()
    {
        if ( canSee(DungeonMaster.class) ) 
        {
            eat(DungeonMaster.class);
            Greenfoot.playSound("au.wav");
            Greenfoot.stop();
        }
    }
}